From 7685c9f7ae67792203234ed5504fdf1446a1aca9 Mon Sep 17 00:00:00 2001
From: Sebastian Droge <sebastian@centricular.com>
Date: Sun, 20 Dec 2015 23:49:35 +0300
Subject: [PATCH] Fix AVC-Intra padding for not Annex B encoding

---
 common/bitstream.c | 21 +++++++++++++++------
 encoder/encoder.c  | 25 +++++++++++--------------
 2 files changed, 26 insertions(+), 20 deletions(-)

diff --git a/common/bitstream.c b/common/bitstream.c
index ec9836a..2373cd4 100644
--- a/common/bitstream.c
+++ b/common/bitstream.c
@@ -80,19 +80,28 @@ void x264_nal_encode( x264_t *h, uint8_t *dst, x264_nal_t *nal )
     *dst++ = ( 0x00 << 7 ) | ( nal->i_ref_idc << 5 ) | nal->i_type;
 
     dst = h->bsf.nal_escape( dst, src, end );
-    int size = (dst - orig_dst) - 4;
+    int size = dst - orig_dst;
+
+    /* Apply AVC-Intra padding */
+    if( h->param.i_avcintra_class )
+    {
+        nal->i_padding -= size - (nal->i_payload + NALU_OVERHEAD);
+        if( nal->i_padding > 0 )
+        {
+            memset( dst, 0, nal->i_padding );
+            size += nal->i_padding;
+        }
+        nal->i_padding = X264_MAX( nal->i_padding, 0 );
+    }
 
     /* Write the size header for mp4/etc */
     if( !h->param.b_annexb )
     {
         /* Size doesn't include the size of the header we're writing now. */
-        orig_dst[0] = size>>24;
-        orig_dst[1] = size>>16;
-        orig_dst[2] = size>> 8;
-        orig_dst[3] = size>> 0;
+        M32( orig_dst ) = endian_fix32( size - 4 );
     }
 
-    nal->i_payload = size+4;
+    nal->i_payload = size;
     nal->p_payload = orig_dst;
     x264_emms();
 }
diff --git a/encoder/encoder.c b/encoder/encoder.c
index c87d79d..56eab42 100644
--- a/encoder/encoder.c
+++ b/encoder/encoder.c
@@ -1958,22 +1958,10 @@ static int x264_encoder_encapsulate_nals( x264_t *h, int start )
 
     for( int i = start; i < h->out.i_nal; i++ )
     {
-        int old_payload_len = h->out.nal[i].i_payload;
         h->out.nal[i].b_long_startcode = !i || h->out.nal[i].i_type == NAL_SPS || h->out.nal[i].i_type == NAL_PPS ||
                                          h->param.i_avcintra_class;
         x264_nal_encode( h, nal_buffer, &h->out.nal[i] );
         nal_buffer += h->out.nal[i].i_payload;
-        if( h->param.i_avcintra_class )
-        {
-            h->out.nal[i].i_padding -= h->out.nal[i].i_payload - (old_payload_len + NALU_OVERHEAD);
-            if( h->out.nal[i].i_padding > 0 )
-            {
-                memset( nal_buffer, 0, h->out.nal[i].i_padding );
-                nal_buffer += h->out.nal[i].i_padding;
-                h->out.nal[i].i_payload += h->out.nal[i].i_padding;
-            }
-            h->out.nal[i].i_padding = X264_MAX( h->out.nal[i].i_padding, 0 );
-        }
     }
 
     x264_emms();
@@ -3841,8 +3829,17 @@ static int x264_encoder_frame_end( x264_t *h, x264_t *thread_current,
         int ret = x264_check_encapsulated_buffer( h, h0, h->out.i_nal, frame_size, frame_size + filler );
         if( ret < 0 )
             return -1;
-        memset( h->out.nal[0].p_payload + frame_size, 0, filler );
-        h->out.nal[h->out.i_nal-1].i_payload += filler;
+        uint8_t *nal_data = h->out.nal[h->out.i_nal-1].p_payload;
+        int nal_size = h->out.nal[h->out.i_nal-1].i_payload;
+        memset( nal_data + nal_size, 0, filler );
+        nal_size += filler;
+        /* Fix up the size header for mp4/etc */
+        if( !h->param.b_annexb )
+        {
+            /* Size doesn't include the size of the header we're writing now. */
+            M32( nal_data ) = endian_fix32( nal_size - 4 );
+        }
+        h->out.nal[h->out.i_nal-1].i_payload = nal_size;
         h->out.nal[h->out.i_nal-1].i_padding = filler;
         frame_size += filler;
     }
-- 
1.9.5.msysgit.0

